#!/usr/bin/env node
var fs = require('fs');

var program = require('commander')
  .version(require('./package.json').version)
  .usage('[options] <file ...>')
  .option('-m, --max <integer>', 'Inlined max resource file size. defaults to 2048', parseInt)
  .option('-f, --force', 'Ignore warns')
  .parse(process.argv);

var files = program.args;
var max = program.max || 2048;
var force = !!program.force;
var parse = require('./inline-css-image');


var i = files.length;
while (i--) {
  try {
    var res = parse(files[i], max);

    if (res.warns.length) {
      res.warns.forEach(function (warn) {
        console.warn(warn.code, warn.error.message, warn.info);
      });
      if (!force) break;
    }

    fs.writeFileSync(files[i], res.content);
    console.log('\nWrite file ' + files[i] + ' successfully!');

  } catch (e) {
    if (!force) throw e;
  }
}


