var fs = require('fs');
var path = require('path');
var fileType = require('file-type');
var reResource = /(src=|url\(\s*['"]?)([^'"\)\(\?\|#)]+)(['"]?\s*\)?)/gm;
var reComment = /\/\*[\s\S]*?\*\//gm;

// 文件中有注释怎么办 ？
// 有可能两个地方用同一张图片
// data:image/png;base64,iVBOR...
module.exports = function parse(styleFile, maxSize, filter) {
  var content = fs.readFileSync(styleFile, 'utf8');
  var map = {};

  var reduplicateFiles = [];
  var warns = [];
  var commentZones = [];

  content.replace(reComment, function (raw, index) {
    commentZones.push([index, index + raw.length]);
  });

  content = content.replace(reResource, function (raw, prefix, src, suffix, index) {
    var stats, filepath, filterResult;

    if (/^\w+:/.test(src)) return raw;
    if (isIndexInZones(index, commentZones)) return raw;

    filepath = path.resolve(path.dirname(styleFile), src);

    var info = {filepath: filepath, raw: raw, src: src, index: index};

    if (typeof filter === 'function') filterResult = filter(info);
    if (filterResult === false) return raw;

    try {
      stats = fs.statSync(filepath);
      if (!stats.isFile()) throw new Error('Resource is not file');
    } catch (e) {
      warns.push({code: 102, error: e, info: info});
      return raw;
    }

    if (filterResult !== true && stats.size > maxSize) return raw;

    if (map[filepath]) {
      if (reduplicateFiles.indexOf(filepath) < 0) reduplicateFiles.push(filepath);
      map[filepath].push(info);
    } else {
      map[filepath] = [info];
    }

    var buf = fs.readFileSync(filepath);
    var ft = fileType(buf);
    if (!ft) {
      warns.push({code: 103, error: new Error('Can not find resource file type'), info: info});
      return raw;
    }
    return prefix + 'data:' + ft.mime + ';base64,' + fs.readFileSync(filepath).toString('base64') + suffix;
  });

  if (reduplicateFiles.length) {
    warns.push({
      code: 101,
      error: new Error('Reduplicate resource files'),
      info: reduplicateFiles.reduce(function (memo, key) {
        memo[key] = map[key];
        return memo;
      }, {})
    })
  }

  return {warns: warns, content: content};
}


function isIndexInZones(index, zones) {
  var i = zones.length;
  while (i--) {
    if (index >= zones[i][0] && index <= zones[i][1]) return true;
  }
  return false;
}

